{% extends "notifications/email_template.html" %}

{% block body %}
<p style="font-size: 20px; color: #313952; text-align: center;"><strong>Dear {{profile.first_name}} {{profile.last_name}}</strong></p>
<p style="text-align: center;">Attached you find the requested Excel-Report for {{customer.search_name}}</p>
{% endblock %}
